#ifndef QT_FEATURES_QuickShapes_src_quickshapes_qtquickshapes_config_h_H
#define QT_FEATURES_QuickShapes_src_quickshapes_qtquickshapes_config_h_H


#endif // QT_FEATURES_QuickShapes_src_quickshapes_qtquickshapes_config_h_H
