/* This file was generated by syncqt. */
#ifndef QT_QTQMLWORKERSCRIPT_VERSION_H
#define QT_QTQMLWORKERSCRIPT_VERSION_H

#define QTQMLWORKERSCRIPT_VERSION_STR "6.11.0"

#define QTQMLWORKERSCRIPT_VERSION 0x0601100

#endif // QT_QTQMLWORKERSCRIPT_VERSION_H
