// Copyright (C) 2016 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only

#ifndef QTQUICKWIDGETSGLOBAL_H
#define QTQUICKWIDGETSGLOBAL_H

#include <QtCore/qglobal.h>
#include <QtQuickWidgets/qtquickwidgetsexports.h>

#endif // QTQUICKWIDGETSGLOBAL_H
