/* This file was generated by syncqt. */
#ifndef QT_QTLABSSTYLEKITIMPL_VERSION_H
#define QT_QTLABSSTYLEKITIMPL_VERSION_H

#define QTLABSSTYLEKITIMPL_VERSION_STR "6.11.0"

#define QTLABSSTYLEKITIMPL_VERSION 0x0601100

#endif // QT_QTLABSSTYLEKITIMPL_VERSION_H
