// Copyright (C) 2021 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
// Qt-Security score:insignificant

#ifndef QTLABSSHAREDIMAGEGLOBAL_P_H
#define QTLABSSHAREDIMAGEGLOBAL_P_H

#include <QtCore/qglobal.h>

//
//  W A R N I N G
//  -------------
//
// This file is not part of the Qt API.  It exists purely as an
// implementation detail.  This header file may change from version to
// version without notice, or even be removed.
//
// We mean it.
//

#include <QtLabsSharedImage/qtlabssharedimageexports.h>

QT_BEGIN_NAMESPACE

void Q_LABSSHAREDIMAGE_EXPORT qml_register_types_Qt_labs_sharedimage();

QT_END_NAMESPACE

#endif // QTLABSSHAREDIMAGEGLOBAL_P_H
